from django.test import TestCase

from django.contrib.auth import get_user_model

from django_omise.models.core import Customer
from django_omise.models.schedule import ChargeSchedule
from django_omise.models.choices import Currency, SchedulePeriod


from django.utils import timezone

from unittest import mock

from .test_utils import (
    mocked_requests_post,
    mocked_requests_get,
    mocked_add_card_request,
    mocked_delete_card_request,
)

User = get_user_model()

# Create your tests here.
class CustomerTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username="test_user1", email="test_user1@email.com"
        )

        self.post_patcher = mock.patch(
            "requests.post", side_effect=mocked_requests_post
        )
        self.get_patcher = mock.patch("requests.get", side_effect=mocked_requests_get)

        self.post_patcher.start()
        self.get_patcher.start()

        self.customer, self.created = Customer.get_or_create(user=self.user)
        self.customer.sync_cards()

    def tearDown(self):
        mock.patch.stopall()

    def test_charge_schedule_human_amount(self):
        charge_schedule = ChargeSchedule.objects.create(
            id="test_charge_schedule_id",
            amount=100000,
            currency=Currency.THB,
            livemode=False,
            card=self.customer.cards.live().first(),
            customer=self.customer,
            default_card=False,
        )

        self.assertEqual(charge_schedule.human_amount, "1,000.00")

    def test_charge_schedule_human_amount_jpy(self):
        charge_schedule = ChargeSchedule.objects.create(
            id="test_charge_schedule_id",
            amount=100000,
            currency=Currency.JPY,
            livemode=False,
            card=self.customer.cards.live().first(),
            customer=self.customer,
            default_card=False,
        )

        self.assertEqual(charge_schedule.human_amount, "100,000.00")



from django_omise.models.schedule import Schedule
from django_omise.models.core import Customer
from django_omise.omise import omise

from django_omise.tests.mockdata.charge import (
    base_charge_response,
    base_charge_with_metadata_response,
)

from django_omise.tests.test_utils import (
    mocked_base_charge_request,
    mocked_set_charge_metadata_request,
    mocked_schedule_without_next_occurrences_on_request,
)

from django_omise.tests.base import OmiseBaseTestCase

from unittest import mock

# Create your tests here.
class BaseModelTestCase(OmiseBaseTestCase):
    def test_class_get_field_names(self):
        field_names = [field.name for field in Customer.get_field_names()]
        fields = Customer._meta.get_fields()
        for field in fields:
            self.assertIn(field.name, field_names)

    def test_class_get_field_name_with_ignored_fields(self):
        test_field = Customer._meta.get_fields()[0]
        field_names = [
            field.name
            for field in Customer.get_field_names(ignore_fields=[test_field.name])
        ]
        self.assertNotIn(test_field.name, field_names)

    @mock.patch(
        "requests.get", side_effect=mocked_schedule_without_next_occurrences_on_request
    )
    def test_build_defaults_with_default_value(self, mock_get_schedule):
        schedule = omise.Schedule.retrieve("test_schedule_id")
        defaults = Schedule.build_defaults_from_omise_object(
            omise_object=schedule,
            ignore_fields=[
                "occurrences",
                "customer",
                "charge",
            ],
        )
        self.assertEqual(defaults.get("next_occurrences_on"), [])

    @mock.patch("requests.patch", side_effect=mocked_set_charge_metadata_request)
    @mock.patch("requests.get", side_effect=mocked_base_charge_request)
    def test_set_metadata_with_none(self, mock_get_charge, mock_set_metadata):
        metadata = None
        django_charge = self.create_charge()
        django_charge.set_metadata(metadata=metadata)

        mock_set_metadata.assert_called_once()

        self.assertEqual(django_charge.metadata, dict())

    @mock.patch("requests.patch", side_effect=mocked_set_charge_metadata_request)
    @mock.patch("requests.get", side_effect=mocked_base_charge_request)
    def test_set_metadata_with_data(self, mock_get_charge, mock_set_metadata):
        metadata = {"data": "data"}
        django_charge = self.create_charge()
        django_charge.set_metadata(metadata=metadata)

        mock_set_metadata.assert_called_once()

        self.assertEqual(django_charge.metadata, metadata)

    @mock.patch("omise.Charge.retrieve")
    def test_get_omise_object(self, mock_omise_retrieve):
        charge = self.create_charge()
        charge.get_omise_object()
        mock_omise_retrieve.assert_called_once_with(charge.id)

    @mock.patch(
        "django_omise.models.base.OmiseBaseModel.update_or_create_from_omise_object"
    )
    @mock.patch("requests.get", side_effect=mocked_base_charge_request)
    def test_reload_from_omise(self, mock_get_charge, mock_update_or_create_method):
        charge = self.create_charge()
        customer = self.create_customer(id="cust_test_5s1jz157366mu6wr0ng")
        charge.reload_from_omise()
        mock_update_or_create_method.assert_called_once()

    def test_str_method(self):
        charge = self.create_charge()
        self.assertIsInstance(str(charge), str)



import json

from django.urls import reverse
from django_omise.tests.base import ClientAndUserBaseTestCase, OmiseBaseTestCase

from django_omise.tests.mockdata.event import schedule_with_one_charge_event_response
from django_omise.tests.test_utils import mocked_requests_event_schedule

from django_omise.omise import omise


from unittest import mock


class ViewTestCase(ClientAndUserBaseTestCase, OmiseBaseTestCase):
    def setUp(self):
        self.customer = self.create_customer(id="test_customer_id")
        self.card = self.create_card(id="test_card_id")

    def test_webhook_no_json_format_status_code(self):
        response = self.client.post(
            reverse("django_omise:webhook"),
            "This is justa raw body",
            content_type="text/plain",
        )
        self.assertEqual(response.status_code, 400)

    @mock.patch(
        "omise.Event.retrieve",
        side_effect=omise.errors.NotFoundError("event not found"),
    )
    def test_webhook_no_event_found(self, mock_event_retrieve):
        response = self.client.post(
            reverse("django_omise:webhook"),
            schedule_with_one_charge_event_response,
            content_type="application/json",
        )
        self.assertEqual(response.status_code, 404)

    @mock.patch("requests.get", side_effect=mocked_requests_event_schedule)
    def test_webhook_schedule(self, mock_get_event):
        response = self.client.post(
            reverse("django_omise:webhook"),
            schedule_with_one_charge_event_response,
            content_type="application/json",
        )
        self.assertEqual(response.status_code, 200)



from django.test import TestCase

from django_omise.models.core import Source
from django_omise.models.choices import Currency

# Create your tests here.
class SourceTestCase(TestCase):
    def test_source_human_amount(self):
        source = Source(amount=100000, currency=Currency.THB)
        self.assertEqual(source.human_amount, "1,000.00")

    def test_source_human_amount_jpy(self):
        source = Source(amount=100000, currency=Currency.JPY)
        self.assertEqual(source.human_amount, "100,000.00")

    def test_source_type_name_not_exists(self):
        source = Source(amount=100000, currency=Currency.JPY, type="not_real_source")
        with self.assertRaises(KeyError):
            source.source_type_name



from django.contrib.auth import get_user_model

from django_omise.models.core import Customer, Card, Charge
from django_omise.models.choices import Currency
from django_omise.omise import omise
from django_omise.utils.core_utils import (
    get_model_from_omise_object,
    is_omise_object_instances,
    update_or_create_from_omise_object_action,
)

from django_omise.tests.base import OmiseBaseTestCase

from unittest import mock


from .test_utils import (
    mocked_requests_post,
    mocked_requests_get,
    mocked_base_charge_request,
    mocked_base_customer_request,
)

User = get_user_model()

# Create your tests here.
class CoreUtilTestCase(OmiseBaseTestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username="test_user1", email="test_user1@email.com"
        )

        self.post_patcher = mock.patch(
            "requests.post", side_effect=mocked_requests_post
        )
        self.get_patcher = mock.patch("requests.get", side_effect=mocked_requests_get)

        self.post_patcher.start()
        self.get_patcher.start()

        self.customer, self.created = Customer.get_or_create(user=self.user)
        self.customer.sync_cards()

    def tearDown(self):
        mock.patch.stopall()

    def test_get_model_from_omise_object_customer(self):
        customer = omise.Customer.retrieve("test_customer_id")
        self.assertEqual(get_model_from_omise_object(omise_object=customer), Customer)

    def test_get_model_from_omise_object_card(self):
        customer = omise.Customer.retrieve("test_customer_id")
        card = customer.cards[0]
        self.assertEqual(get_model_from_omise_object(omise_object=card), Card)

    @mock.patch("requests.post", side_effect=mocked_base_charge_request)
    def test_get_model_from_omise_object_charge(self, mock_charge_request):
        charge = omise.Charge.create(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first().id,
        )

        self.assertEqual(get_model_from_omise_object(omise_object=charge), Charge)

    def test_get_model_from_omise_object_not_implemented(self):
        new_object = type("test", (object,), {})()
        new_object.object = "new_object"
        self.assertEqual(get_model_from_omise_object(omise_object=new_object), None)

    def test_get_model_from_omise_object_not_implemented_raises(self):
        new_object = type("test", (object,), {})()
        new_object.object = "new_object"
        with self.assertRaises(ValueError):
            get_model_from_omise_object(
                omise_object=new_object, raise_if_not_implemented=True
            )

    @mock.patch("requests.get", side_effect=mocked_base_customer_request)
    def test_is_omise_instance_charge(self, mock_get_charge):
        customer = omise.Customer.retrieve("test_customer_id")
        self.assertTrue(is_omise_object_instances(customer))

    def test_update_or_create_with_non_omise_object(self):
        non_omise_object = type("test", (), {})()
        self.assertIsNone(
            update_or_create_from_omise_object_action(omise_object=non_omise_object)
        )



# Generated by Django 3.2.11 on 2022-05-23 01:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_omise', '0003_alter_source_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charge',
            name='status',
            field=models.CharField(choices=[('failed', 'Failed'), ('expired', 'Expired'), ('pending', 'Pending'), ('reversed', 'Reversed'), ('successful', 'Successful'), ('unknown', 'Unknown')], help_text='Status of this charge.', max_length=10),
        ),
        migrations.AlterField(
            model_name='source',
            name='charge_status',
            field=models.CharField(choices=[('failed', 'Failed'), ('expired', 'Expired'), ('pending', 'Pending'), ('reversed', 'Reversed'), ('successful', 'Successful'), ('unknown', 'Unknown')], help_text='Status of charge created using this source (if any)', max_length=10),
        ),
    ]
